/** \file read.h
  \brief Functions that are responsible for reading and printing
    input, arguments and other stuff.
    
   This file contains functions that read arguments and settings,
   prints them. There are functions that initialize and free necessary
   structs.

*/

/**   \fn char *get_line( FILE *fp )
   \brief Read a line from a file.

   \arg \p fp Filepointer to an opended (at least for reading) file.

   The function \b get_line() reads a line of arbitrary length from the file
   pointed to by \p fp. The file should be opened (at least) for
   reading before calling \b get_line().

   \return \b get_line() returns the read line as string.
*/

/**\fn struct gotoh_arguments gotoh_readArguments( int argc, char *argv[] )
   \brief Read the command line arguments.

   \arg \c argc Number of command line arguments
   \arg \c argv Array containing the command line arguments

   \b gotoh_readArguments() reads the command line arguments and
   initializes the program respectively. If something is wrong with
   the input help messages are returned and the program exits.

   \return \b gotoh_readArguments() returns a struct containing the
   values for each argument.
   
*/

/**\fn struct gotoh_settings gotoh_readSettings( gotoh_arguments ga )
   \brief Initialize settings.

   \arg \c ga Struct containing the command line arguments.

   \b gotoh_readSettings() parses the command line arguments, checks
   the validity of each parameter and exits the program if something
   is wrong with the parameters.

   \return \b gotoh_readSettings() returns a struct containing the
   values for all settings.
*/

/**\fn struct gotoh_motifSingle gotoh_readMotifSingle( char *file )
   \brief Read a motif into a data structure.

   \arg \c file The filename of the file containing the motif as frequency pattern.

   \b gotoh_readMotifSingle() reads the frequency pattern of the query
   motif into a datastruct. Additionally, it stores min and max values
   for each column of the motif as well as the information content.

   \return \b gotoh_readMotifSingle() returns a struct containing the
   matrices for frequency, min, max and information content.
*/

/**\fn struct gotoh_motif gotoh_readMotif2sided(char *file, gotoh_settings cfg)
   \brief Read a 2-sided motif into a data structure.

   \arg \c file The filename of the file containing the motif as
   frequency patterns with list of paired positions.
   \arg \c cfg Struct containing the configuration settings

   \b gotoh_readMotif2sided() reads the frequency patterns of both
   sides of the motif and the corresponding base pair information. For
   details on the format see \c scripts/reformat.pl.

   \return \b gotoh_readMotif2sided() returns a struct containing all
   information about the motif.
*/

/**\fn gotoh_arguments gotoh_initArguments()
   \brief Initialize the arguments struct.

   \return \b gotoh_initArguments() returns an initialized struct for
   the command line arguments.
*/

/**\fn gotoh_settings gotoh_initSettings()
   \brief Initialize the settings struct.

   \return \b gotoh_initSettings() returns an initialized struct for
   the settings.
*/

/**\fn void gotoh_printArguments(gotoh_arguments ga)
   \brief Print arguments struct.

   \arg \c ga Struct for command line arguments
*/

/**\fn void gotoh_printSettings(gotoh_settings gs)
   \brief Print settings struct.

   \arg \c gs Struct for settings
*/

/**\fn void gotoh_printMotifSingle(struct gotoh_motifSingle m)
   \brief Print the input motif.

   \arg \c m Struct containing all motif information
*/

/**\fn void gotoh_printMotif(struct gotoh_motif m)
   \brief Print the input Motif (2-sided).

   \arg \c m Struct containing all motif information
*/

/**\fn void usage()
   \brief Print usage information
*/

/**\fn void version()
   \brief Print version information
*/

/**\fn void gotoh_freeArguments(gotoh_arguments ga)
   \brief Free arguments struct.

   \arg \c ga Struct containing the command line arguments.
*/

/**\fn void gotoh_freeSettings(gotoh_settings gs)
   \brief Free settings struct.

   \arg \c gs Struct for settings
*/

/**\fn void gotoh_freeMotifSingle(gotoh_motifSingle gm)
   \brief Free struct for motif.

   \arg \c gm Struct containing all motif information
*/

/**\fn void gotoh_freeMotif(gotoh_motif gm)
   \brief Free struct for 2-sided motif.

   \arg \c gm Struct containing all motif information
*/

/**\fn void gotoh_Error(char *msg, int v)
   \brief Print error message and exit program.

   \arg \c msg Message as string
   \arg \c v Verbose value
*/

/**\fn void gotoh_Warning(char *msg, int v)
   \brief Print warning to stderr, continue program.

   \arg \c msg Message as string
   \arg \c v Verbose value
*/

#ifndef READ_H
#define READ_H

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "structs.h"

char *get_line(FILE *fp);

struct gotoh_arguments gotoh_readArguments(int argc, char *argv[]);

struct gotoh_settings gotoh_readSettings(gotoh_arguments ga);

struct gotoh_motifSingle gotoh_readMotifSingle(char *file);

struct gotoh_motif gotoh_readMotif2sided(char *file, gotoh_settings cfg);

/* init functions */
struct gotoh_arguments gotoh_initArguments();

struct gotoh_settings gotoh_initSettings();

/* print functions */
void gotoh_printArguments(gotoh_arguments ga);

void gotoh_printSettings(gotoh_settings gs);

void gotoh_printMotifSingle(struct gotoh_motifSingle m);

void gotoh_printMotif(struct gotoh_motif m);

void usage();

void version();

/* free functions */
void gotoh_freeArguments(gotoh_arguments ga);

void gotoh_freeSettings(gotoh_settings gs);

void gotoh_freeMotifSingle(gotoh_motifSingle gm);

/* void gotoh_freeMotif2sided(gotoh_motif gm); */

void gotoh_freeMotif(gotoh_motif gm);

/* other functions */
void gotoh_Error(char *msg, int v);

void gotoh_Warning(char *msg, int v);

#endif
