/** \file structs.h
  \brief Structs that help to organize data.
    
   This file contains the definition of all structs needed by
   GotohScan. The respective initializing and free functions are
   distributed over the src/x.h files.x
*/

/** \struct iarray
    \brief Contains an array of Integer numbers and its size.

    \b iarray Contains an array of Integer numbers and its size. It
    can be initialized with the function \b gotoh_initArrayI() and
    freed with \b gotoh_freeArrayI().

*/

/** \struct farray
    \brief Contains an array of Float numbers and its size.

    \b farray Contains an array of Integer numbers and its size. It
    can be initialized with the function \b gotoh_initArrayF() and
    freed with \b gotoh_freeArrayF().
*/

/** \struct gotoh_arguments
    \brief Contains all command line arguments.

    \b gotoh_arguments contains all command line arguments or their
    initialized values if not set by the user. It can be initialized
    by \b gotoh_initArguments() and freed by \b gotoh_freeArguments().
*/

/** \struct gotoh_settings
    \brief Contains all settings.

    \b gotoh_settings contains all settings needed by GotohScan. Unset
    values are set to appropriate defaults. It can be initialized by
    \b gotoh_initSettings() and freed by \b gotoh_freeSettings().
*/

/** \struct gotoh_aln
    \brief Stores information about one alignment

    \b gotoh_aln contains all information about a single
    alignment. Alignment scores and positions in the database are
    computed during the forward recursion, while the alignment is
    obtained in the backward recursion. The E-value is calculated only
    after the complete set of database sequences has been scanned.
*/

/** \struct gotoh_motif
    \brief Holds the input 2-sided motif.

    \b gotoh_motif holds the frequency matrices and the base pair
    table of a 2-sided motif. Additionally, it stores min, max,
    information content and consensus as well as maximal possible
    scores for sequence and structure.
*/

/** \struct gotoh_motifSingle
    \brief Holds the input motif.
    
    \b gotoh_motif holds the frequency matrix of a
    motif. Additionally, it stores min, max, information content of
    the frequency matrix and the consensus character.
*/

/** \struct gotoh_motifCandidate
    \brief Holds the computed scoring of a candidate for a matching 2-sided motif.

    \b gotoh_motifCandidate holds the aligned consensus sequences of
    pattern and database, the corresponding scores for sequence and
    structure for each stem and the complete score
*/

#ifndef STRUCTS_H
#define STRUCTS_H

/* -------------------------------------------------------------------------- */

struct iarray {

  int *vals;   /**< Array of Integer */
  int *start;    /**< Array of start positions */
  int size;    /**< Number of elements in the Integer array */
  
} iarray;

/* -------------------------------------------------------------------------- */

struct farray {

  float *vals; /**< Array of Float */
  int *start;    /**< Array of start positions */
  int size;    /**< Number of elements in the Float array */
  
} farray;

/* -------------------------------------------------------------------------- */

struct darray {

  double *vals; /**< Array of Double */
  int size;     /**< Number of elements in the Float array */
  
} darray;

/* -------------------------------------------------------------------------- */

typedef struct gotoh_arguments {

  char *cfg_file;     /**< Filename of configuration file. Format see \e examples/Configuration.dat. */
  char *db_file;      /**< Filename of database file containing FASTA formatted sequences. */
  char *q_file;       /**< Filename of query file conatining FASTA formatted (ungapped) sequences. */
  char *res_file;     /**< filenmae of output file. */ 
  char *stat_file;    /**< Filename of file where to statistics on the
			 E-value calculation, i.e. the data of the
			 histogram, the fit parameters and the final
			 fitted curve */
  char *m1_file;      /**< Filename of query file containing a motif
			 as frequency matrix reformatted using \b
			 scripts/reformatFasta.pl */
  char *m2_file;      /**< Filename of query file containing a 2-sided
			 motif as frequency matrices reformatted using
			 \b scripts/reformatFasta.pl */
  int mD;             /**< Maximal distance between the 2 sides of the 2-sided motif */
  double e;           /**< Evalue - only candidate alignments with
			 lower or equal Evalue than this value are
			 returned. \e Default: \c 1e-3*/
  int p;              /**< Percent identity - only candidate
			 alignments with higher or equal percent
			 identity than this value are returned. \e Default: \c 60*/
  int out;            /**< Output format. Values are:
			 \arg \c 0 - Blast tabular output
			 \arg \c 1 - Blast tabular output + aligned sequences
			 \arg \c 2 - FASTA format. NOTE: Hit sequence only, without gaps !
			 \arg \c 3 - MAF format. NOTE: Header truncated to 30 characters!
			 \arg \c 4 - BED + aligned sequences
			 \arg \c 5 - GFF + aligned sequences

			 \e Default: \c 0
		      */
  int help;           /**< Print usage information. */
  int version;        /**< Print version information. */
  int verbose;        /**< Print Verbose output. */
  int quiet;          /**< Don't brint Status */
  float m1_seqT;      /**< Sequence threshold to be a matching pattern
			 for single motifs. \e Default: \c 0.7 */
  float m2_seqT;      /**< Sequence threshold to be a matching pattern
			 for 2-sided motifs. \e Default: \c 0.5 */
  float m2_strucT;    /**< Structure threshold to be a matching
			 pattern for 2-sided motifs. \e Default: \c
			 0.8 */
  float m2T;          /**< Complete Threshold for 2-sided motifs. \e
			 Default: \c 0.6 */
  int noE;            /**< Tells the program if the user wants to get
			 all results - not only those with a higher
			 Evalue than the defined one. */
  
} gotoh_arguments;

/* -------------------------------------------------------------------------- */

typedef struct gotoh_settings {

  /* input */
  char *dbase;        /**< Filename of database file containing FASTA formatted sequences. */
  char *query;        /**< Filename of query file conatining FASTA formatted (ungapped) sequences. */
  char *res_file;     /**< filenmae of output file. */ 
  char *motif1;       /**< Filename of query file containing a motif as
			 frequency matrix reformatted using \b
			 scripts/reformatFasta.pl */
  char *motif2;       /**< Filename of query file containing a 2-sided
			 motif as frequency matrices reformatted using
			 \b scripts/reformatFasta.pl */
  
  /* alignment parameters / scores */
  int gap_open;       /**< Penalty for opening a gap. \e Default: \c -8 */
  int gap_extend;     /**< Penalty for extending an existing gap. \e Default: \c -2 */
  int match;          /**< Bonus for a matching pair of bases. \e Default: \c 3 */
  int mismatch;       /**< Penalty for a pair of bases that does not match. \e Default: \c -1 */
  int inf;            /**< The definition of infinity which is used to
			 initialize the scoring arrays of the Gotoh
			 algorithm. \e Default: \c 999999 */
  /* filter */
  int maxdistance;    /**< Maximal distance between the 2 sides of the 2-sided motif */
  double usrE;        /**< Evalue - only candidate alignments with
			 lower or equal Evalue than this value are
			 returned. \e Default: \c 1e-3*/
  float percID;       /**< Percent identity - only candidate
			 alignments with higher or equal percent
			 identity than this value are returned. \e Default: \c 60*/
  float m1_seqT;      /**< Sequence threshold to be a matching pattern
			 for single motifs. \e Default: \c 0.7 */
  float m2_seqT;        /**< Sequence threshold to be a matching pattern
			 for 2-sided motifs. \e Default: \c 0.5 */
  float m2_strucT;    /**< Structure threshold to be a matching
			 pattern for 2-sided motifs. \e Default: \c
			 0.8 */
  float m2T;          /**< Complete Threshold for 2-sided motifs. \e
			 Default: \c 0.6 */
  /* output */
  int output_format;  /**< Output format. Values are:
			 \arg \c 0 - Blast tabular output
			 \arg \c 1 - Blast tabular output + aligned sequences
			 \arg \c 2 - FASTA format. NOTE: Hit sequence only, without gaps !
			 \arg \c 3 - MAF format. NOTE: Header truncated to 30 characters!
			 \arg \c 4 - BED + aligned sequences
			 \arg \c 5 - GFF + aligned sequences

			 \e Default: \c 0
		      */
  char *stat_file;    /**< Filename of file where to statistics on the
			 E-value calculation, i.e. the data of the
			 histogram, the fit parameters and the final
			 fitted curve */
  int verbose;        /**< Print Verbose output. */
  int quiet;          /**< Don't brint Status */

  
} gotoh_settings;

/* -------------------------------------------------------------------------- */

/* stores all information for one alignment */
typedef struct gotoh_aln {

  int db_len;            /**< Length of complete database sequence -> for maf output */
  char *hitseq;          /**< Aligned database (hit) sequence (with gaps) */
  char *queryseq;        /**< Aligned query sequence/pattern (with gaps) */
  char *hitname;         /**< Name of database entry */
  char *queryname;       /**< Name of query sequence/pattern */
  int shift;             /**< Score to shift x-axis to zero */
  int sStart;            /**< Start position in database entry */
  int sEnd;              /**< End position in database entry */
  int length;            /**< Length of alignment */
  char strand;           /**< Strand of hit in database */
  float alnscore;        /**< Alignment score */
  float ntfalnscore;     /**< Unshifted (real) alignment score */
  double evalue;         /**< E-value for alignment score */
  float percID;          /**< Percent identity. Number of real matches
			    vs. number of maximal possible matches */
  int mismatches;        /**< Number of mismatching pairs of bases */
  int gapopening;        /**< Number of gap opening events */
  int qStart;            /**< Start of alignment in query. Should
			    always be 0 due to global alignment */
  int qEnd;              /**< End of alignment in query. Should always
			    be sizeof(query) due to global
			    alignment */

  
} gotoh_aln;

/* -------------------------------------------------------------------------- */

typedef struct gotoh_motif {

  char *comment;         /**< Comment for the motif, provided in input file. */
  char *name;            /**< Name of the motif. So far, same as comment. */
  int nLeft;             /**< The size of the left stem of a 2-sided motif. */
  int nRight;            /**< The size of the right stem of a 2-sided motif. */
  int len;               /**< Minimal length of motif. \e Default: \c nLeft + \c nRight + \c 3 */
  float **stemL;         /**< Frequency table of left stem. */
  float **stemR;         /**< Frequency table of right stem. */
  float *minL;           /**< Minimal values of \c stemL. */
  float *maxL;           /**< Maximal values of \c stemL. */
  float *infoL;          /**< Information content of \c stemL. */
  int *maxCL;            /**< Number of base at max value in \c
			    stemL. {\c 0,1,2,3,4} for {\c
			    A,C,G,U,-}. */
  char *consL;           /**< Base at \c maxCL - Consensus sequence of left stem. */
  float *minR;           /**< Minimal values of \c stemR. */
  float *maxR;           /**< Maximal values of \c stemR. */
  float *infoR;          /**< Information content of \c stemR. */
  int *maxCR;            /**< Number of base at max value in \c
			    stemR. {\c 0,1,2,3,4} for {\c
			    A,C,G,U,-}. */
  char *consR;           /**< Base at \c maxCR - Consensus sequence of left stem. */
  int n_bp;              /**< Number of base pairs. */
  int **basepairs;       /**< Positions of paired bases in their stems and base pair
			    type. 3 columns:
			    \arg \c 0 - i (left)
			    \arg \c 1 - j (right)
			    \arg \c 2 - type */
  float scoreBP;        /**< Structure score consensus motif */
  float maxScore;       /**< Complete score of consensus motif:
			   sum((sum(maxL), sum(maxR), scoreBP)) */
  
} gotoh_motif;

/* -------------------------------------------------------------------------- */

typedef struct gotoh_motifSingle {

  char *name;           /**< Name of motif. */
  int len;              /**< Length of motif. */
  float **freq;         /**< Frequency table of motif. */
  float *min;           /**< Minimal values of \c freq. */
  float *max;           /**< Maximal values of \c freq. */
  float *info;          /**< Information content \c freq. */
  char *maxC;           /**< Number of base at max value in \c
			    freq. {\c 0,1,2,3,4} for {\c
			    A,C,G,U,-}. */
  
} gotoh_motifSingle;

/* -------------------------------------------------------------------------- */

typedef struct gotoh_motifCandidate {

  char *left_seq;       /**< Aligned database (hit) sequence of left stem (with gaps). */
  char *left_mseq;      /**< Aligned motif (consensus) sequence of left stem (with gaps). */
  char *right_seq;      /**< Aligned database (hit) sequence of right stem (with gaps). */
  char *right_mseq;     /**< Aligned motif (consensus) sequence of right stem (with gaps). */
  float scoreL;         /**< Sequence alignment score of left stem. */
  float scoreL_perc;    /**< Percent identiy of left stem alignment. */
  float scoreR;         /**< Sequence alignment score of right stem. */
  float scoreR_perc;    /**< Percent identiy of right stem alignment. */
  float scoreBP;        /**< Base pair score. */
  float scoreBP_perc;   /**< Real base pair score vs. maximal possible
			   base pair score of motif. Note: this can be
			   higher than 100, since some structures
			   might get a better score than the one of
			   the motif. */
  float score;          /**< Overall score. (\c scoreL + \c scoreR + \c scoreBP) / \c 3 */
  float score_perc;     /**< Real overall score vs. maximal possible
			   score of motif. Note: This score may exceed
			   100, due to a better structure score than
			   the motif has. */
  int sL;               /**< Start of left stem in database. */
  int eL;               /**< End of left stem in database. */
  int sR;               /**< Start of right stem in database. */
  int eR;               /**< End of right stem in database. */
  int distance;         /**< Distance between both stems in database. */

} gotoh_motifCandidate;

#endif
